#
# multcdf.R, 11 Jun 23
#
# For derivations of CDF see:
# https://shape-of-code.com/2023/04/16/distribution-of-binary-operator-results/
# and Wolfram Alpha
# https://www.wolframalpha.com/input?i2d=true&i=Integrate%5BDivide%5BPower%5B%5C%2840%29log+x%5C%2841%29%2C%5C%2840%29n-1%5C%2841%29%5D%2C%5C%2840%29n-1%5C%2841%29%21%5D%2Cx%5D

library("expint")
library("colorspace")
library("plyr")


par(bty="l")
par(las=1)
par(pch="+")
pal_col=rainbow(9)


cdf_line=function(N, col_str)
{
lines(x, multcdf(N, x), col=col_str)

x_pos=1e-6
text(x_pos, multcdf(N, x_pos)*1.2, label=N)
}


multcdf=function(n, x)
{
return (gammainc(n, -log(x))/factorial(n-1))
}


x=seq(1e-9, 1e-3, by=1e-8)

plot(x, multcdf(3, x), log="xy", type="l",, col=pal_col[1],
	yaxs="i",
	xlim=c(1e-8, 1e-3), ylim=c(1e-6, 1.0),
	xlab="X", ylab="Probability less than X")

cdf_line(3, pal_col[1])
cdf_line(4, pal_col[2])
cdf_line(5, pal_col[3])
cdf_line(6, pal_col[4])
cdf_line(8, pal_col[5])
cdf_line(10, pal_col[6])
cdf_line(12, pal_col[7])
cdf_line(14, pal_col[8])
cdf_line(16, pal_col[9])


