#
# pv-fishing.R, 11 Jun 23

library("colorspace")
library("plyr")


par(bty="l")
par(las=1)
par(pch="+")
pal_col=rainbow(4)


plot_opnum=function(op_val)
{
t=subset(nop, oprnd == op_val)
t=t[order(t$opnum), ]

plot(1, type="n", log="y",
	xaxs="i",
	xlim=c(0, op_val), ylim=range(t$ndead),
	xlab="Perturb point", ylab="Number unchanged")

legend("topleft", legend=paste0(c(20 , 40 , 60 , 80 ), "% add"),
                        fill=pal_col, border="white", bty="n")

d_ply(subset(t, addmult==0.2), .(nesting), function(df) lines(df$opnum, df$ndead, col=pal_col[1]))
d_ply(subset(t, addmult==0.4), .(nesting), function(df) lines(df$opnum, df$ndead, col=pal_col[2]))
d_ply(subset(t, addmult==0.6), .(nesting), function(df) lines(df$opnum, df$ndead, col=pal_col[3]))
d_ply(subset(t, addmult==0.8), .(nesting), function(df) lines(df$opnum, df$ndead, col=pal_col[4]))
}


plot_nest_opnum=function(op_val, nest)
{
op_lines=function(df, col_str)
   {
   lines(df$opnum, df$ndead/max_pt_dead, col=col_str)
   }

t=subset(nop, (oprnd == op_val) & (nesting == nest))
t=t[order(t$opnum), ]

op_lines(subset(t, addmult==0.2), col=pal_col[1])
op_lines(subset(t, addmult==0.4), col=pal_col[2])
op_lines(subset(t, addmult==0.6), pal_col[3])
op_lines(subset(t, addmult==0.8), pal_col[4])
}


max_pt_dead=200*200 # Number of expressions * numer of iterations

all_pv=read.csv("allpv.csv", as.is=TRUE)
nexpr=read.csv("allexprpv.csv", as.is=TRUE)
nop=read.csv("alloppv.csv", as.is=TRUE)
nop=subset(nop, opnum != 0)

pert=read.csv("all_pert.csv")

# Expressions containing more operands enable more perturbation possibilities
# Model the normalised count of 'dead' perturbations
nexpr$norm_expr=nexpr$nexpr/(nexpr$oprnd-1)

plot(~ addmult+nesting+oprnd+nexpr, data=nexpr)

plot(~ addmult+nesting+oprnd+opnum+ndead, data=nop)

plot_opnum(150)
plot_opnum(125)
plot_opnum(100)
plot_opnum( 75)
plot_opnum( 50)
plot_opnum( 25)

plot(1, type="n", log="y",
	xaxs="i", yaxs="i",
	xlim=c(0, 100), ylim=c(1e-5, 1),
	xlab="Perturb point", ylab="Fraction unchanged")

legend("topleft", legend=paste0(100-c(20 , 40 , 60 , 80 ), "% multiply"),
                        fill=pal_col, border="white", bty="n")

plot_nest_opnum(100, 0.75)
plot_nest_opnum( 50, 0.75)

# npv_nls=nls(norm_expr ~ a+b*addmult^c, data=nexpr, trace=TRUE, start=c(a=0.66, b=-0.7, c=1))
# summary(npv_nls)
# On its own, addmult^1.5 is a better fit than addmult,
# but not a better fit when other variables are included.
# The following did not converge
# npv_nls=nls(norm_expr ~ a+b*nesting^c, data=nexpr, trace=TRUE, start=c(a=0.33, b=-0.09, c=1))

# float
npv_mod=glm(norm_expr ~ sqrt(oprnd)*addmult-addmult+nesting, data=nexpr)
summary(npv_mod)

# double
npv_mod=glm(norm_expr ~ sqrt(oprnd)*(addmult+nesting), data=nexpr)
summary(npv_mod)

pert$nc_norm=pert$nochange/pert$oprnd
pert$nc_frac=pert$nochange/(pert$oprnd*max_pt_dead)

plot( ~ nochange+nc_frac+addmult+nest+oprnd+log(pert), data=pert)

pert_mod=glm(nc_frac ~ (addmult+sqrt(oprnd))*log(pert)-log(pert)+addmult:sqrt(oprnd), data=pert)
summary(pert_mod)

pert_mod=glm(nc_norm ~ addmult+sqrt(oprnd)+log(pert), data=pert)
summary(pert_mod)

